/*
    mkicon - Microsoft Windows Icon File Generator
    Copyright (C) 2003  Michael Boyle

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
using System;
using System.Drawing;
using System.IO;
using System.Reflection;
using System.Threading;

[assembly:AssemblyTitle("mkicon"),AssemblyDescription("Microsoft Windows Icon File Generator")]
namespace mkicon
{
	struct BitmapInfoHeader
	{
		public int Size;
		public int Width;
		public int Height;
		public short Planes;
		public short BitCount;
		public int Compression;
		public int SizeImage;
		public int XPelsPerMeter;
		public int YPelsPerMeter;
		public int ClrUsed;
		public int ClrImportant;
		public BitmapInfoHeader(int width, int height)
		{
			this.Size = 40;
			this.Width = width;
			this.Height = 2 * height;
			this.Planes = 1;
			this.BitCount = 32;
			this.Compression = 0;
			this.SizeImage = 0; //width * 4 * height;
			this.XPelsPerMeter = 0;
			this.YPelsPerMeter = 0;
			this.ClrUsed = 0;
			this.ClrImportant = 0;
		}
		public void Write(System.IO.BinaryWriter bw)
		{
			bw.Write(this.Size);
			bw.Write(this.Width);
			bw.Write(this.Height);
			bw.Write(this.Planes);
			bw.Write(this.BitCount);
			bw.Write(this.Compression);
			bw.Write(this.SizeImage);
			bw.Write(this.XPelsPerMeter);
			bw.Write(this.YPelsPerMeter);
			bw.Write(this.ClrUsed);
			bw.Write(this.ClrImportant);
		}
	}
	struct NewHeader
	{
		public short Reserved;
		public short ResType;
		public short ResCount;
		public NewHeader(int rescount)
		{
			this.Reserved = 0;
			this.ResType = 1;
			this.ResCount = ((short) rescount);
		}
		public void Write(System.IO.BinaryWriter bw)
		{
			bw.Write(this.Reserved);
			bw.Write(this.ResType);
			bw.Write(this.ResCount);
		}
	}
	struct IconResDir
	{
		public byte Width;
		public byte Height;
		public byte ColorCount;
		public byte reserved;
		public IconResDir(int width, int height)
		{
			this.Width = ((byte) width);
			this.Height = ((byte) height);
			this.ColorCount = 0;
			this.reserved = 0;
		}
		public void Write(System.IO.BinaryWriter bw)
		{
			bw.Write(this.Width);
			bw.Write(this.Height);
			bw.Write(this.ColorCount);
			bw.Write(this.reserved);
		}
	}
	struct ResDir
	{
		public IconResDir Icon;
		public short Planes;
		public short BitCount;
		public int BytesInRes;
		public int Offset;
		public ResDir(int width, int height)
		{
			this.Icon = new IconResDir(width, height);
			this.Planes = 1;
			this.BitCount = 32;
			this.BytesInRes = 40 + ((((((width + 7) >> 3) + 3) >> 2) << 2) + (width * 4)) * height;
			this.Offset = 0;
		}
		public void Write(System.IO.BinaryWriter bw)
		{
			this.Icon.Write(bw);
			bw.Write(this.Planes);
			bw.Write(this.BitCount);
			bw.Write(this.BytesInRes);
			bw.Write(this.Offset);
		}
	}
	class MainClass
	{
		static unsafe byte[] GetBitmapBits(System.Drawing.Bitmap bm)
		{
			System.Drawing.Imaging.BitmapData bd = bm.LockBits(new System.Drawing.Rectangle(0, 0, bm.Width, bm.Height), System.Drawing.Imaging.ImageLockMode.ReadWrite, System.Drawing.Imaging.PixelFormat.Format32bppArgb);
			byte* bi  = ((byte*) ((void*) bd.Scan0));
			byte[] bo = new byte[bd.Stride * bd.Height];
			for(int y = 0; y < bd.Height; y++)
			{
				int i = (bd.Height - y - 1) * bd.Stride;
				int o = y * bd.Stride;
				for(int x = bd.Stride - 1; x >= 0; x--)
				{
					bo[o + x] = bi[i + x];
				}
			}
			bm.UnlockBits(bd);
			return bo;
		}
		public static void Main(string[] args)
		{
			if(args.Length < 2)
			{
				Console.WriteLine("Microsoft Windows Icon File Generator\nsyntax: mkicon output.ico files...");
				return;
			}
			System.Drawing.Bitmap[] bm = new System.Drawing.Bitmap[args.Length - 1];
			for(int i = 1; i < args.Length; i++)
			{
				bm[i - 1] = ((System.Drawing.Bitmap) System.Drawing.Bitmap.FromFile(args[i]));
			}
			using(System.IO.BinaryWriter bw = new System.IO.BinaryWriter(new System.IO.FileStream(args[0], System.IO.FileMode.Create, System.IO.FileAccess.Write)))
			{
				NewHeader nh = new NewHeader(bm.Length);
				ResDir[] rd = new ResDir[bm.Length];
				BitmapInfoHeader[] bmih = new BitmapInfoHeader[bm.Length];
				byte[][] bits = new byte[bm.Length][];
				byte[][] masks = new byte[bm.Length][];
				for(int i = 0; i < bm.Length; i++)
				{
					int stride = 4 * bm[i].Width;
					int maskStride = ((((bm[i].Width + 7) >> 3) + 3) >> 2) << 2;
					rd[i] = new ResDir(bm[i].Width, bm[i].Height);
					bmih[i] = new BitmapInfoHeader(bm[i].Width, bm[i].Height);
					bits[i] = GetBitmapBits(bm[i]);
					masks[i] = new byte[maskStride * bm[i].Height];
					for(int y = bm[i].Height - 1; y >= 0; y--)
					{
						int ofs = y * stride;
						int maskOfs = y * maskStride;
						for(int x = bm[i].Width - 1; x >= 0; x--, ofs += 4)
						{
							if(bits[i][ofs + 3] < 128)
							{
								bits[i][ofs + 3] = 0;
								bits[i][ofs + 2] = 0;
								bits[i][ofs + 1] = 0;
								bits[i][ofs] = 0;
								masks[i][maskOfs + (x >> 3)] |= ((byte) (0x80 >> (x & 7)));
							}
						}
					}
				}
				rd[0].Offset = 6 + 16 * bm.Length;
				for(int i = 1; i < bm.Length; i++)
				{
					rd[i].Offset = rd[i - 1].Offset + rd[i - 1].BytesInRes;
				}
				nh.Write(bw);
				for(int i = 0; i < bm.Length; i++)
				{
					rd[i].Write(bw);
				}
				for(int i = 0; i < bm.Length; i++)
				{
					bmih[i].Write(bw);
					bw.Write(bits[i]);
					bw.Write(masks[i]);
				}
			}
		}
	}
}
